import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext } from '../utils/index.js'
import * as d3 from 'd3-selection'
import { drag as d3Drag } from 'd3-drag'


const Draggable = ({
    children = null,
    onDragStart,
    onDragEnd
}) => {
    const {canvas, simulation} = useContext(GraphContext)
    const drag = d3Drag()

    useEffect(() => {
        const container = d3.select(canvas.current)

        drag.on('start', (...args) => {
            if (onDragStart) onDragStart(d3.event)
            if (!d3.event.active) simulation.alphaTarget(0.3).restart()
            d3.event.subject.fx = d3.event.subject.x
            d3.event.subject.fy = d3.event.subject.y
            container.style('cursor', 'grabbing')
        })
    
        drag.on('drag', (...args) => {
            d3.event.subject.fx = d3.event.x
            d3.event.subject.fy = d3.event.y
        })
    
        drag.on('end', (...args) => {
            if (onDragEnd) onDragEnd(d3.event)
            if (!d3.event.active) simulation.alphaTarget(0)
            d3.event.subject.fx = null
            d3.event.subject.fy = null
            container.style('cursor', 'grab')
        })

        drag.subject(() => {
            const { transform } = simulation
            const x = transform.invertX(d3.event.x)
            const y = transform.invertY(d3.event.y)
            const node = simulation.find(x, y, 20)

            return node
        })

        drag.container(canvas.current)
        d3.select(canvas.current).call(drag)
    })

    return children
}

Draggable.propTypes = {
    children: PropTypes.node
}

export default Draggable