import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, isVisible } from '../utils/index.js'


const Links = ({
    stroke = '#ccc',
    width = 1.5
}) => {
    const {canvas, simulation, links, bounds} = useContext(GraphContext)

    useEffect(() => {
        const ctx = canvas.current.getContext('2d')

        simulation.on('tick.links', () => {
            const { transform } = simulation

            ctx.save()

            if (transform) {
                ctx.translate(transform.x, transform.y)
                ctx.scale(transform.k, transform.k)
            }

            ctx.beginPath()

            links.forEach(d => {
                if (
                    isVisible(d.source, transform, bounds) ||
                    isVisible(d.target, transform, bounds)
                ) {
                    ctx.moveTo(d.source.x, d.source.y)
                    ctx.lineTo(d.target.x, d.target.y)
                }
            })

            ctx.lineWidth = width
            ctx.strokeStyle = stroke
            ctx.stroke()
            ctx.restore()
        })
    })

    return null
}

Links.propTypes = {
    attributes: PropTypes.object,
    style: PropTypes.object,
    onTick: PropTypes.func
}

export default Links