import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, getRelativePosition } from '../utils/index.js'
import { zoom as d3Zoom, zoomIdentity } from 'd3-zoom'
import * as d3 from 'd3-selection'


const Zoomable = ({
    scale = 1,
    scaleExtent = [0.5, 1],
    children = null
}) => {
    const {canvas, simulation, bounds} = useContext(GraphContext)
    const zoom = d3Zoom()

    useEffect(() => {
        const container = d3.select(canvas.current)

        container.style('cursor', 'move')

        zoom.filter(() => {
            const {transform} = simulation
            const {x, y} = getRelativePosition(d3.event.x, d3.event.y, canvas.current)
            const transformed = transform.invert([x, y])
            
            return !d3.event.shiftKey && !simulation.find(...transformed, 20)
        })

        zoom.on('start', () => {
            simulation.alphaTarget(0.1).restart()
        })

        zoom.on('zoom', () => {
            simulation.transform = d3.event.transform
        })

        zoom.on('end', () => {
            simulation.alphaTarget(0)
        })

        zoom.scaleExtent(scaleExtent)
        container.call(zoom)
        zoom.transform(container, zoomIdentity.translate(
            (bounds.width - bounds.width * scale) / 2,
            (bounds.height - bounds.height * scale) / 2
        ).scale(scale))
    })
    
    return children
}

Zoomable.propTypes = {
    scale: PropTypes.number,
    scaleExtent: PropTypes.array,
    children: PropTypes.node,
    onZoom: PropTypes.func
}

export default Zoomable