import React from 'react'
import { renderToStaticMarkup } from 'react-dom/server'

/**
 * A cache mapping resource types to canvas-friendly image data
 * 
 * @type {object}
 */
const iconDataCache = {}

/**
 * Convert a react component into an image element and data that's compatible
 * with our canvas-based graph.
 *
 * @param  {array}  definitions - Settings for each type of node.
 * @param  {object} d           - Data backing a given node.
 * @return {object}             - An image `element` and some metadata.
 */
export const renderIcon = (definitions, d) => {
    if (!iconDataCache[d.type]) {
        const definition = definitions.find(obj => obj.type === d.type)
        
        if (definition && definition.icon) {
            const Icon = definition.icon
            const image = new Image()
            const svg = renderToStaticMarkup(
                <Icon
                    xmlns="http://www.w3.org/2000/svg"
                    version="1.1"
                    width={24}
                    height={24}
                    shadowLength={0}
                />
            )

            image.src = `data:image/svg+xml;utf8,${encodeURIComponent(svg)}`

            iconDataCache[d.type] = {
                element: image,
                x: -12,
                y: -12,
                width: 24,
                height: 24
            }
        }
    }

    return iconDataCache[d.type]
}