import { createContext } from 'react'

/**
 * Exposes top-level graph data, simulation, and more.
 * 
 * @type {object}
 */
export const GraphContext = createContext({
    simulation: null,
    nodes: null,
    links: null
})

/**
 * Account for canvas offset in the given x and y coordinates.
 *
 * @param  {number} x 	   - X coordinate
 * @param  {number} y 	   - Y coordinate
 * @param  {object} canvas - A canvas element
 * @return {object} 	   - The offset `x` and `y` coordinates
 */
export const getRelativePosition = (x, y, canvas) => {
    const {top, left} = canvas.getBoundingClientRect()

    return {
        x: x - left,
        y: y - top
    }
}

/**
 * Determine whether the given `node` is visible within the `bounds`.
 *
 * @param  {object}  node      - A valid graph node (containing `x` / `y` coordinates)
 * @param  {object}  transform - A d3 transform instance
 * @param  {object}  bounds    - The `width` and `height` of the graph
 * @return {boolean}           - Indicates whether the node is visible
 */
export const isVisible = (node, transform, bounds) => {
    const {width, height} = bounds
    const [x, y] = transform.apply([node.x, node.y])

    return  x > 0 && x < width && y > 0 && y < height
}